package org.rosuda.jrs;

import static javax.script.ScriptEngine.ENGINE;
import static javax.script.ScriptEngine.ENGINE_VERSION;
import static javax.script.ScriptEngine.LANGUAGE;
import static javax.script.ScriptEngine.LANGUAGE_VERSION;
import static javax.script.ScriptEngine.NAME;

import java.lang.reflect.Constructor;
import java.util.Collections;
import java.util.List;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;

import org.rosuda.REngine.REngine;
import org.rosuda.REngine.REngineCallbacks;

/**
 * R implementation of ScriptEngineFactory.
 * @author Martyn Tebby
 */
public class RScriptFactory implements ScriptEngineFactory {

	private static final String R_ENGINE_NAME = "REngine";
	private static final String R_ENGINE_VERSION = "0.7";
	private static final String R_LANGUAGE_NAME = "R";
	private static final String R_LANGUAGE_VERSION = "2";
	private static final String R_FILE_EXTENSION = "R";
	private static final List<String> R_MIME_TYPES = null;

	/** Name of the property used to set the default host. */
	public static final String HOST_PROPERTY = "org.rosuda.jrs.host";

	/** Default host to use for Rserve connection, or JRI if null. */
	public static String defaultHost = System.getProperty(HOST_PROPERTY);

	/** Host to use for Rserve connection, or JRI if null. */
	public String host = defaultHost;

	/** Name of the property used to run the REPL for JRI. */
	public static final String RUN_REPL_PROPERTY = "org.rosuda.jrs.runrepl";

	/** Default run the R REPL, required for output redirection in JRI. */
	public static boolean defaultRunREPL = Boolean.getBoolean(RUN_REPL_PROPERTY);

	/**
	 * Run the R REPL, required for output redirection in JRI.
	 * No Effect with Rserve.
	 * Causes JRI to run slower and not to close.
	 */
	public boolean runREPL = defaultRunREPL;

	@Override
	public String getEngineName() {
		return R_ENGINE_NAME;
	}

	@Override
	public String getEngineVersion() {
		return R_ENGINE_VERSION;
	}

	@Override
	public String getLanguageName() {
		return R_LANGUAGE_NAME;
	}

	@Override
	public String getLanguageVersion() {
		return R_LANGUAGE_VERSION;
	}

	@Override
	public List<String> getNames() {
		return Collections.singletonList(R_ENGINE_NAME);
	}

	@Override
	public List<String> getExtensions() {
		return Collections.singletonList(R_FILE_EXTENSION);
	}

	@Override
	public List<String> getMimeTypes() {
		return R_MIME_TYPES;
	}

	@Override
	public Object getParameter(String key) {
		if(key == null)						return null;
		if(key.equals(ENGINE))				return getEngineName();
		if(key.equals(ENGINE_VERSION))		return getEngineVersion();
		if(key.equals(LANGUAGE))			return getLanguageName();
		if(key.equals(LANGUAGE_VERSION))	return getLanguageVersion();
		if(key.equals(NAME))				return getNames().get(0);
		return null;
	}

	@Override
	public String getMethodCallSyntax(String obj, String m, String... args) {
		String ret = m + "(" + obj;
		for(String arg : args) {
			ret += "," + arg;
		}
		ret += ");";
		return ret;
	}

	@Override
	public String getOutputStatement(String toDisplay) {
		return "print('" + toDisplay + "');";
	}

	@Override
	public String getProgram(String... statements) {
		String ret = "";
		for(String str : statements) {
			ret += str + ";\n";
		}
		return ret;
	}

	@Override
	public ScriptEngine getScriptEngine() {
		try {
			boolean closeREPL = host == null && runREPL;
			RScriptEngine engine = new RScriptEngine(closeREPL);
			REngine rengine = createREngine(engine);
			engine.setEngine(rengine);
			engine.setFactory(this);
			return engine;
		}
		catch(Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	// Use reflection to minimize jars needed to build and run
	private REngine createREngine(REngineCallbacks callbacks) throws Exception {
		if(host != null) {
			Class<?> cls = Class.forName("org.rosuda.REngine.Rserve.RConnection");
			Constructor<?> ctr = cls.getDeclaredConstructor(String.class);
			return (REngine) ctr.newInstance(host);
		}
		else {
			String cls = "org.rosuda.REngine.JRI.JRIEngine";
			String[] args = { "--vanilla", "--slave" };
			return REngine.engineForClass(cls, args, callbacks, runREPL);
		}
	}

}
